%% This script constructs the narrative factors as well as their differences
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Define the path to data files and the save results path 

savePath    = '.\results';
loadPath    = '.\data';

%% Define the numbers of days to look back, i.e, days back from time t which is 
%% the monetary policy announcement date. Results will be computed for each of 
%% numbers of days listed, i.e., 1 day, 1-2 days, 1-3 days, etc. 
daysToLookBack      = 1 : 10;
                        
%% Read table with counts for minutes and media. The counts are based on the
%% vocabulary used in the minutes. These data are partly proprietary (the news data)
%% Thus, data is not provided. However, the code is constructed to read a .csv file
%% with the following format: type, date, word 1, word2, etc. 
%% type is either:
%% 'ebd' - for MP communnication reports 
%% 'news' - for news articles
%% date har the format - '1999-10-27 00:00:00'
%% The entries for word 1, word 2, etc. are word counts. One entry for each article. 
%%
TC = readtable(fullfile(loadPath, 'countMatrix.csv'), 'ReadVariableNames', true, 'FileEncoding', 'UTF-8');

%% Remove last row, which is nan data (in our special case, you might need to change this depending on the csv file
TC = TC(1:end-1, :);
    

%% Remove weekend from the news data
[DayNumber,DayName] = weekday(TC.Date);
weekendsIdx         = ismember(DayNumber, [7 1]); % Saturday and Sunday
TC                  = TC(weekendsIdx == 0, :);

%% Construct factor name lists to use when rotating the SVD solution
%% (Note: This list is in Norwegian in this application)                                                 
 factorNameIdList    = { 
                         {'inflasjonen'};
                         {'oljeprisen'};
                         {'kronekursen'};
                         {'arbeidsledigheten'};                         
                         {'usikkerheten'};
                         {'kreditten'};
                       };                  

% Useful if you have more names for each narrative dimension cell                   
factorNameIdListShort = arrayfun(@(x) x{1}{1}, factorNameIdList, 'UniformOutput',false);

%% Read table with count of sentiment words. This table has the same structure as 
%% the TC table above. However, word columns (with counts) only include words from the
%% postive(negative) sentiment dictionary. 
T_sentimentPositive = readtable(fullfile(loadPath,'positive_words.csv'), 'ReadVariableNames', false, 'FileEncoding', 'UTF-8');
T_sentimentNegative = readtable(fullfile(loadPath,'negative_words.csv'), 'ReadVariableNames', false, 'FileEncoding', 'UTF-8');

%% Replace some strange Norwegian letter....replace , ,  with _
sentimentPositiveList = T_sentimentPositive.Var1;
sentimentPositiveList = strrep(sentimentPositiveList, '', '_');
sentimentPositiveList = strrep(sentimentPositiveList, '', '_');
sentimentPositiveList = strrep(sentimentPositiveList, '', '_');
sentimentNegativeList = T_sentimentNegative.Var1;
sentimentNegativeList = strrep(sentimentNegativeList, '', '_');
sentimentNegativeList = strrep(sentimentNegativeList, '', '_');
sentimentNegativeList = strrep(sentimentNegativeList, '', '_');

%% Collapse columns for each category in factorNameIdList into one
%%(if you have more than one word for each narrative dimension)
t_vname             = TC.Properties.VariableNames(3:end);
t_x                 = TC{:, 3 : end};
for i = 1 : size(factorNameIdList, 1)            
    [~, ~, ib]  = intersect(factorNameIdList{i}, t_vname, 'stable');    
    % Use mean here so that the number of counts does not become affected
    % by number of synonyms 
    tmp             = mean(t_x(:, ib), 2);
    t_x(:, ib)      = [];
    t_vname(ib)     = [];    
    t_x             = cat(2, tmp, t_x);
    t_vname         = cat(2, factorNameIdList{i}(1), t_vname);           
end
TCC = array2table(t_x, 'VariableNames', t_vname);
TCC = cat(2, array2table(TC.Date, 'VariableNames', {'Date'}), TCC);
TCC = cat(2, array2table(TC.Type, 'VariableNames', {'Type'}), TCC);
TC  = TCC;

%% Remove monetary policy reports from TC
mpr_idx                 = strcmpi(TC.Type, 'mpr');
TC(mpr_idx, :)          = [];

%% Sort the count matrices, and contruct the factors from the normalized count matrixes
t_dates = TC.Date;
t_x     = TC{:, 3 : end};
t_type  = TC.Type;
t_vname = TC.Properties.VariableNames(3:end);

eba_idx = find(strcmpi(t_type', {'ebd'}) == 1);
% Extract eba (minutes)
eba_dates = t_dates(eba_idx); 
eba_x     = t_x(eba_idx, :);

%% Get positive and negative words
[positiveSentimentTerms, sia, sib] = intersect(sentimentPositiveList, t_vname);
[negativeSentimentTerms, nia, nib] = intersect(sentimentNegativeList, t_vname);

%% Run the loop over different dauys to look back
for db = daysToLookBack    
    %% Extract news
    [news_bx, news_ax]              = deal(nan(numel(eba_idx), numel(t_vname)));
              
    for i = 1 : numel(eba_idx)        
         news_bx(i, :)          = mean( t_x(eba_idx(i) - (db + 1) : eba_idx(i)-1, :) , 1);                          
         if (db + 1)  + eba_idx(i) + 1 > size(t_x, 1)
             news_ax(i : end, :)        = 0.0001;                               
         else
             news_ax(i, :)          = mean( t_x(eba_idx(i) + 1 : (db + 1)  + eba_idx(i) + 1, :), 1);             
         end            
    end

    %% Get sentiment of each row
    eba_xn_sentiment                        = (sum(eba_x(:, sib), 2) - sum(eba_x(:, nib), 2));
    eba_xn_sentiment(eba_xn_sentiment == 0) = 0.0001;
    news_bxn_sentiment                        = (sum(news_bx(:, sib), 2) - sum(news_bx(:, nib), 2));
    news_bxn_sentiment(news_bxn_sentiment == 0) = 0.0001;    
    news_axn_sentiment                        = (sum(news_ax(:, sib), 2) - sum(news_ax(:, nib), 2));
    news_axn_sentiment(news_axn_sentiment == 0) = 0.0001;            

    %% normalize both 
     eba_xn      = eba_x ./ repmat(sum(eba_x, 2), [1 numel(t_vname)]);
     news_bxn    = news_bx ./ repmat(sum(news_bx, 2), [1 numel(t_vname)]);
     news_axn    = news_ax ./ repmat(sum(news_ax, 2), [1 numel(t_vname)]);

    %% Do tf-idf on the eba set and cut off words with very low score from this set for both sets                      
    n                = size(eba_xn, 1);
    idfj             = log(n./sum(eba_xn > 0, 1));
    
    eba_xn          = eba_xn.*repmat(idfj, [n 1]);
    news_bxn        = news_bxn.*repmat(idfj, [n 1]);
    news_axn        = news_axn.*repmat(idfj, [n 1]);
        
    %% Re-order variables such that they appear in order of factorNameIdListShort
    [cc, ia, ib]        = intersect(factorNameIdListShort, t_vname, 'stable');

    % re-order variable ordering for both matrices
    eba_xn_id           = eba_xn(:, ib);
    news_bxn_id         = news_bxn(:, ib);
    news_axn_id         = news_axn(:, ib);
    eba_xn(:, ib)       = [];   
    news_bxn(:, ib)     = [];   
    news_axn(:, ib)     = [];   
    
    % For computation
    eba_xn_id           = zscore([eba_xn_id eba_xn]);
    news_bxn_id         = zscore([news_bxn_id news_bxn]);
    news_axn_id         = zscore([news_axn_id news_axn]);        
    
    %% Compute factors for each data set
    numberOfFactors         = numel(factorNameIdList);
    
    [U, S, V]               = svd(eba_xn_id, 'econ');
    loadings_svd_eba        = V(:, 1 : numberOfFactors);
    factors_svd_eba         = U(:, 1 : numberOfFactors)*S(1:numberOfFactors,1:numberOfFactors);
    % Do the rotation
    % words x factors
    H                       = loadings_svd_eba(1 : numberOfFactors, :);    
    factors_svd_id_eba      = factors_svd_eba*H';

    [U, S, V]               = svd(news_bxn_id, 'econ');
    loadings_svd_news       = V(:, 1 : numberOfFactors);
    factors_svd_news        = U(:, 1 : numberOfFactors)*S(1:numberOfFactors,1:numberOfFactors);
    % Do the rotation
    % words x factors
    H                       = loadings_svd_news(1 : numberOfFactors, :);
    factors_svd_id_news     = factors_svd_news*H';

    [U, S, V]               = svd(news_axn_id, 'econ');
    loadings_svd_news_a     = V(:, 1 : numberOfFactors);
    factors_svd_news_a      = U(:, 1 : numberOfFactors)*S(1:numberOfFactors,1:numberOfFactors);
    % Do the rotation
    % words x factors
    H                       = loadings_svd_news_a(1 : numberOfFactors, :);    
    factors_svd_id_news_a   = factors_svd_news_a*H';
                
    %% Compare eba to news, one pair at the time
    n                           = size(eba_xn_id, 1);       
    sumDiffTimeSeries_indi      = nan(n, 2);    
    sumDiffTimeSeries_indi_ba   = nan(n, 2);        
    
    for i = 1 : n
        % Using the individually computed factors, from the two sets                        
        sumDiffTimeSeries_indi(i, 1)    = sum((factors_svd_id_eba(i,:) - factors_svd_id_news(i, :)).^2);                
        sumDiffTimeSeries_indi_ba(i, 1) = sum((factors_svd_id_news(i,:) - factors_svd_id_news_a(i, :)).^2);    
    end
    
    
    %% Save the stuff
    save( fullfile(savePath, ['identifiedFactors_db' int2str(db) '.mat']), 'eba_dates', 'factors_svd_id_eba', 'factors_svd_id_news', 'factorNameIdListShort',...
                 'eba_xn_sentiment', 'news_bxn_sentiment',  'news_axn_sentiment', 'sumDiffTimeSeries_indi', 'sumDiffTimeSeries_indi_ba');    
   
end

